@@ -1,17 +1,18 @@
 /*
-    BINIO.H
-    Declare structures and functions for arrays of data that have
-    meaningful interpretations on disk.
-
-    $Id: binio.h,v 1.1 1993/08/27 18:32:09 munro Exp $
-
-    Yorick supports the following data types:
-
-       char   short   int   long   float   double
-       string   pointer   complex   struct(instance)
+ * $Id: binio.h,v 1.3 2010/02/28 21:52:28 dhmunro Exp $
+ * Declare structures and functions for arrays of data that have
+ * meaningful interpretations on disk.
+ *
+ * Yorick supports the following data types:
+ *
+ *     char   short   int   long   float   double
+ *     string   pointer   complex   struct(instance)
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #ifndef BINIO_H
 #define BINIO_H
@@ -29,15 +30,15 @@
          name with its data type, dimensions, and disk address.
       3. An optional block of history information, detailing the
          disk addresses (and files, if this is a family of files) of
-	 the history records, if there are any.
+         the history records, if there are any.
       4. An optional list of pointees, associating memory and disk
          addresses of indirectly referenced data (not explicitly
-	 in the dataTable).
+         in the dataTable).
       5. An optional file containing the (plain text) CLOG description
          of this file.
       6. The list of cache blocks associated with this file, and the
          last address read or written (since ftell doesn't know about
-	 the cache buffers).
+         the cache buffers).
  */
 typedef struct IOStream IOStream;
 
@@ -103,7 +104,7 @@ typedef struct Operations Operations;
    therefore split out to allow this binary file package to be used
    in non-Yorick based codes.  These three values are explicitly
    needed:  */
-extern Operations *yOpsStruct, *yOpsStream, *yOpsStructDef;
+PLUG_API Operations *yOpsStruct, *yOpsStream, *yOpsStructDef;
 
 /* DataLayout is a complete description of a primitive data type,
    including byte size, alignment, byte order, and floating point layout
@@ -131,10 +132,10 @@ typedef struct PointeeList PointeeList;
 typedef struct MDinfo MDinfo;  /* for PointeeList */
 struct PointeeList {
   int writing;         /* 0 if table keyed to disk address,
-			  1 if keyed to memory address */
+                          1 if keyed to memory address */
   long nValid;         /* number of valid objects (i.e.- where both
-			  disk data address and m pointer contain
-			  valid data) */
+                          disk data address and m pointer contain
+                          valid data) */
   HashXTable table;    /* generic hash table for address correspondence */
   MDinfo *mdInfo;      /* memory-disk correspondence list */
 };
@@ -152,33 +153,33 @@ struct IOStream {
   int references;      /* reference counter */
   Operations *ops;     /* virtual function table */
   void *stream;        /* usually the FILE*, but opaque here --
-			  0 indicates file has been closed */
+                          0 indicates file has been closed */
   char *fullname;      /* filename after YExpandName */
   int permissions;     /* +1 read permission, +2 write permission
-			  +4 append mode, +8 binary mode */
+                          +4 append mode, +8 binary mode */
 
   IOOperations *ioOps; /* virtual functions to read and write file */
 
   long blockSize;         /* MUST be a power of 2 minus 1 (default 0x3fff)
-			     all cache blocks will be multiples of
-			     blockSize+1 bytes */
+                             all cache blocks will be multiples of
+                             blockSize+1 bytes */
   CacheBlock *blockList;  /* list of cache blocks for this file,
-			     stored highest to lowest address */
+                             stored highest to lowest address */
   long seqAddress;        /* address immediately after last address
-			     actually read or written using YcRead or
-			     YcWrite -- different from ftell(stream)
-			     because of caching */
+                             actually read or written using YcRead or
+                             YcWrite -- different from ftell(stream)
+                             because of caching */
 
   int structAlign;        /* byte offset of any struct in another struct
-			     must be multiple of this (the alignment of
-			     some structs may be more strict than this) */
+                             must be multiple of this (the alignment of
+                             some structs may be more strict than this) */
   int dataAlign;          /* 0  -- align variables to addresses as if
-			           the file were a giant struct instance
-			     >0 -- byte offset of any variable in file
-			           must be a multiple of this
-			     WARNING WARNING WARNING WARNING WARNING
-			     This MUST be 0 in order to correctly write
-			     a PDB file with history records.  */
+                                   the file were a giant struct instance
+                             >0 -- byte offset of any variable in file
+                                   must be a multiple of this
+                             WARNING WARNING WARNING WARNING WARNING
+                             This MUST be 0 in order to correctly write
+                             a PDB file with history records.  */
 
   HashTable structTable;  /* table of StructDef names for this file */
   StructDef **structList; /* corresponding to structTable->names */
@@ -190,20 +191,25 @@ struct IOStream {
   long offset;            /* to be applied to all addresses */
 
   HistoryInfo *history;   /* non-0 if there is a history record structure
-			     -- If so, then no changes to the dataTable
-			     (which represents static variables) are
-			     permitted.
-			     Note that this IOStream may be either
-			     history->parent or history->child.  */
+                             -- If so, then no changes to the dataTable
+                             (which represents static variables) are
+                             permitted.
+                             Note that this IOStream may be either
+                             history->parent or history->child.  */
 
   CLbuffer *contentsLog;  /* if non-0, contains "CLOG" textual equivalent
-			     of structTable, dataTable, and history */
+                             of structTable, dataTable, and history */
 
   PointeeList pointeeList;  /* list of memory<->disk address correspondence
-			       to aid in pointee reads or writes */
+                               to aid in pointee reads or writes */
   void (*CloseHook)(IOStream *file);  /* if non-0, called before close */
 };
 
+/* check to see if this is in-memory file created by vopen
+ * return value is the Array* or 0, stream is p_file*
+ */
+PLUG_API void *y_vopen_file(void *stream);
+
 /*--------------------------------------------------------------------------*/
 
 /* A Copier copies data of type base from t to s.  If t==s, the Copier
@@ -214,15 +220,15 @@ struct IOStream {
    whose members include pointers or strings.  CopyQ is for stringStruct
    and CopyP is for pointerStruct.  */
 typedef void Copier(StructDef *base, void *dst, const void *src, long n);
-extern Copier CopyX, CopyQ, CopyP, CopyS;
+PLUG_API Copier CopyX, CopyQ, CopyP, CopyS;
 
 /* A Converter converts n objects to or from type base, which will often
    be a foreign disk file data type.  The conversion is from baseData to
    modelData if toBase is 0, from modelData to baseData if toBase
    is non-0.  */
 typedef void Converter(StructDef *base, void *baseData, void *modelData,
-		       long n, int toBase);
-extern Converter ConvertI, ConvertF, ConvertQ, ConvertP, ConvertS;
+                       long n, int toBase);
+PLUG_API Converter ConvertI, ConvertF, ConvertQ, ConvertP, ConvertS;
 
 struct StructDef {
   int references;     /* reference counter */
@@ -241,45 +247,45 @@ struct StructDef {
   IOStream *file;       /* non-zero if the data resides on disk
                            if file!=0, then model!=0 */
   long index;           /* if file!=0, index in file->structList,
-			   else        index in yStructList */
+                           else        index in yStructList */
 
   int addressType;      /* 0 -> object is in memory (file==0), and address
-			        is void*
-			   1 -> object is on disk (file!=0) and address
-			        is long
-			   2 -> address cannot be meaningfully incremented
-			        to find members or array elements  */
+                                is void*
+                           1 -> object is on disk (file!=0) and address
+                                is long
+                           2 -> address cannot be meaningfully incremented
+                                to find members or array elements  */
 
   /* The remaining elements are to implement "transparent translation"
      of this data type into another -- usually to change from disk
      format to memory format.  */
   StructDef *model;     /* memory equivalent of disk StructDef
-			   is NEVER 0 when file!=0,
-			   also destination or source type for Convert */
+                           is NEVER 0 when file!=0,
+                           also destination or source type for Convert */
   Converter *Convert;
          /* Convert may be 0 if no conversion is required.
-	    The order and fpLayout members can be used as a context
-	    by Convert.  In particular, fpLayout need not actually point
-	    to an FPLayout, as long as the first member of the struct
-	    it does point to is a reference counter.  */
+            The order and fpLayout members can be used as a context
+            by Convert.  In particular, fpLayout need not actually point
+            to an FPLayout, as long as the first member of the struct
+            it does point to is a reference counter.  */
   int order;           /* 1 MSB first, -1 LSB first,
-			  -w for LSW first, MSB first within word size w
-			  w for MSW first, LSB first within word size w */
+                          -w for LSW first, MSB first within word size w
+                          w for MSW first, LSB first within word size w */
   FPLayout *fpLayout;  /* if non-0, used by Export/Import to translate
-			  data format */
+                          data format */
 };
 
 /* Nine in-memory data structures are predefined, corresponding with
    the primitive data types in the C programming language:  */
-extern StructDef charStruct;
-extern StructDef shortStruct;
-extern StructDef intStruct;
-extern StructDef longStruct;
-extern StructDef floatStruct;
-extern StructDef doubleStruct;
-extern StructDef stringStruct;   /* char *    always 0-terminated */
-extern StructDef pointerStruct;  /* void *    all pointers generic */
-extern StructDef complexStruct;  /* struct complex { double re, im; } */
+PLUG_API StructDef charStruct;
+PLUG_API StructDef shortStruct;
+PLUG_API StructDef intStruct;
+PLUG_API StructDef longStruct;
+PLUG_API StructDef floatStruct;
+PLUG_API StructDef doubleStruct;
+PLUG_API StructDef stringStruct;   /* char *    always 0-terminated */
+PLUG_API StructDef pointerStruct;  /* void *    all pointers generic */
+PLUG_API StructDef complexStruct;  /* struct complex { double re, im; } */
 /* The indirect types string and pointer are for restricted use only,
    in the sense that special memory management routines must be used
    to allocate and free the associated memory.
@@ -294,11 +300,16 @@ extern StructDef complexStruct;  /* struct complex { double re, im; } */
    yStructList are always, in order, char, short, int, long, float,
    double, string, pointer.  The 9th in yStructList is complex.  */
 
-extern HashTable yStructTable;  /* table of StructDef names for memory */
-extern StructDef **yStructList; /* corresponding to yStructTable->names */
+PLUG_API HashTable yStructTable;  /* table of StructDef names for memory */
+PLUG_API StructDef **yStructList; /* corresponding to yStructTable->names */
 
 /*--------------------------------------------------------------------------*/
 
+/* possibly should do this at configuration time... */
+#if defined(__i386__) || defined(_WIN32)
+# define PAD_ARRAY
+#endif
+
 struct Array {
   int references;   /* reference counter */
   Operations *ops;  /* virtual function table */
@@ -343,14 +354,14 @@ struct Array {
    On PowerPCs, gcc attempts to follow IBMs written rule, and they
    interpret a struct whose first member is an array of double to
    *not* get the extra alignment.  This is yStructAlign=-2.
-   See play/all/numfmt.c for more details.
+   See play/any/numfmt.c for more details.
 
    It is also possible for a C compiler to require different alignment
    for an array than for a scalar.  Yorick cannot handle this.
    Finally, the (hopefully defunct?) hybrid C compiler under the
    (definitely defunct) NLTSS Cray operating system aligned a scalar
    char to an address with remainder 7 modulo 8, which is hopeless.  */
-extern int yStructAlign;
+PLUG_API int yStructAlign;
 
 struct DataLayout {
   int alignment;
@@ -381,7 +392,7 @@ struct FPLayout {
 };
 
 /* Floating point layouts for floats and doubles on this machine */
-extern FPLayout *fltLayout, *dblLayout;
+PLUG_API FPLayout *fltLayout, *dblLayout;
 
 /*--------------------------------------------------------------------------*/
 
@@ -407,30 +418,30 @@ struct IOOperations {
 struct HistoryInfo {
   IOStream *parent;  /* original file always used for static data */
   IOStream *child;   /* separate IOStream used for history record traffic
-			so that child->members[i].base->file has a place
-			to point which may represent a different file in
-			a history family starting from the original parent */
+                        so that child->members[i].base->file has a place
+                        to point which may represent a different file in
+                        a history family starting from the original parent */
 
   int nFamily;       /* number of files in this family */
   char **famNames;   /* used for child->fullname,
-			famNames[0]==parent->fullname
-			allocated length is 4*(1 + (nFamily-1)/4) */
+                        famNames[0]==parent->fullname
+                        allocated length is 4*(1 + (nFamily-1)/4) */
   int fileNumber;    /* 0 to nFamily-1, specifies index of current child */
   long fileSize;     /* approximate maximum size for a file in family */
-#define DEFAULT_FILE_SIZE 0x400000
+#define DEFAULT_FILE_SIZE 0x800000
   int copyParent;    /* non-0 if parent (static variables) is to be copied
-			when a new file is added to this family */
+                        when a new file is added to this family */
 
   long recordSize;   /* length of one record in bytes (not including
-			any strings, pointees, or other indirections) */
+                        any strings, pointees, or other indirections) */
   int recordAlign;   /* alignment for records -- either child->dataAlign,
-			or most restrictive alignment in child->structList
-			when 1st record was declared
-			Unlike a StructDef, recordSize is not necessarily
-			a multiple of recordAlign.  */
+                        or most restrictive alignment in child->structList
+                        when 1st record was declared
+                        Unlike a StructDef, recordSize is not necessarily
+                        a multiple of recordAlign.  */
   long nRecords;     /* number of history records of this type */
   long recNumber;    /* index into ifile and offset
-			for record represented by child (-1 if none) */
+                        for record represented by child (-1 if none) */
 
   /* allocated length of following arrays is 16*(1 + (nRecs-1)/16) */
   int *ifile;        /* list of indices into famNames */
@@ -441,41 +452,44 @@ struct HistoryInfo {
 
 struct MDinfo {
   void *m;       /* memory address, always an Array->value.c; the
-		    MDinfo owns a reference of the Array */
+                    MDinfo owns a reference of the Array */
   long d;        /* disk address of the pointee header */
   long data;     /* disk address of the pointee data */
   StructDef *base;  /* disk StructDef for this data -- this pointer does
-		       NOT own a reference to base */
+                       NOT own a reference to base */
 };
 
 /*--------------------------------------------------------------------------*/
 
 /* fullname is NOT copied and FreeIOStream will do p_free(ios->fullname) */
-extern IOStream *NewIOStream(char *fullname, void *stream, int permissions);
-extern void FreeIOStream(void *ios);      /* *** Use Unref(ios) *** */
+PLUG_API IOStream *NewIOStream(char *fullname, void *stream, int permissions);
+PLUG_API void FreeIOStream(void *ios);      /* *** Use Unref(ios) *** */
 
-extern StructDef *NewStructDef(IOStream *file, long index);
-extern void FreeStructDef(void *base);      /* *** Use Unref(base) *** */
+PLUG_API StructDef *NewStructDef(IOStream *file, long index);
+PLUG_API void FreeStructDef(void *base);      /* *** Use Unref(base) *** */
 
-extern Array *NewArray(StructDef *base, Dimension *dims);
-extern void FreeArray(void *array);        /* *** Use Unref(array) *** */
+PLUG_API Array *NewArray(StructDef *base, Dimension *dims);
+PLUG_API void FreeArray(void *array);        /* *** Use Unref(array) *** */
 
 /* NewTmpArray/ClearTmpArray may be used to get one or two temporary
    arrays.  Call ClearTmpArray first, then NewTmpArray several times,
    then ClearTmpArray.  At the third call to NewTmpArray, the Array
    returned by the first call is Unref'ed; at the fourth call to
    NewTmpArray, the second is Unref'ed; and so on.  */
-extern Array *NewTmpArray(StructDef *base, Dimension *dims);
-extern void ClearTmpArray(void);
+PLUG_API Array *NewTmpArray(StructDef *base, Dimension *dims);
+PLUG_API void ClearTmpArray(void);
 
 /* NewDimension does NOT do Ref(next) */
-extern int yForceOrigin; /* non-zero to ignore stated origin in Dimension
-			    lists, and always assume 1-origin indices */
-extern Dimension *NewDimension(long number, long origin, Dimension *next);
-extern void FreeDimension(Dimension *dims);
+/* WARNING: NewArray(base, NewDimension(...)) is a memory leak!
+ * need to use ynew_dim() below instead in this context
+ */
+PLUG_API int yForceOrigin; /* non-zero to ignore stated origin in Dimension
+                            lists, and always assume 1-origin indices */
+PLUG_API Dimension *NewDimension(long number, long origin, Dimension *next);
+PLUG_API void FreeDimension(Dimension *dims);
 
-extern Strider *NewStrider(long stride, long number);
-extern void FreeStrider(Strider *strider);
+PLUG_API Strider *NewStrider(long stride, long number);
+PLUG_API void FreeStrider(Strider *strider);
 
 /*--------------------------------------------------------------------------*/
 
@@ -484,43 +498,50 @@ extern void FreeStrider(Strider *strider);
    Without this, the Unref macro is unusable.  */
 struct Operations {
   void (*Free)(void *);  /* crucial member for Unref -- first in struct
-			    to allow alternate Operations to be used */
+                            to allow alternate Operations to be used */
 };
 #endif
 
 #define Ref(db) ((db)?++(db)->references:0 , (db))
+#define RefNC(db) (++(db)->references , (db))
 #define Unref(db) {if ((db) && --(db)->references<0) (db)->ops->Free(db);}
+#define UnrefNC(db) {if (--(db)->references<0) (db)->ops->Free(db);}
 
-extern long TotalNumber(const Dimension *dims);
-extern int CountDims(const Dimension *dims);
+PLUG_API long TotalNumber(const Dimension *dims);
+PLUG_API int CountDims(const Dimension *dims);
 /* Note-- like NewDimension, CopyDims does NOT do Ref(next) */
-extern Dimension *CopyDims(Dimension *dims, Dimension *next, int copyOrigin);
+PLUG_API Dimension *CopyDims(Dimension *dims, Dimension *next, int copyOrigin);
 
 /* CopyStrider also does NOT do Ref(next) */
-extern Strider *CopyStrider(Strider *strider, Strider *next);
+PLUG_API Strider *CopyStrider(Strider *strider, Strider *next);
 
 /* tmpDims can be used for "protected" scratch space to build dimension
-   lists.  Before using tmpDims, always call FreeDimension(tmpDims).  */
-extern Dimension *tmpDims;
+ * lists.  Before using tmpDims, always call FreeDimension(tmpDims).
+ * ynew_dim() sets tmpDims to the result of NewDimension(), so
+ * it can be used as argument to functions like NewArray which
+ * increment the use counter
+ */
+PLUG_API Dimension *tmpDims;
+PLUG_API Dimension *ynew_dim(long n, Dimension *next);
 
 /* Given a pointer to data (memory representation of a pointerStruct),
    return the associated Array.  */
-extern void *Pointee(void *pointer);
+PLUG_API void *Pointee(void *pointer);
 
 /* StructEqual tests for two structures which are exactly the same.
    EquivStruct tests whether the file StructDef belonging to an IOStream
    is equivalent to the mem StructDef belonging to yStructTable.  */
-extern int StructEqual(StructDef *l, StructDef *r);
-extern int EquivStruct(StructDef *fil, StructDef *mem);
+PLUG_API int StructEqual(StructDef *l, StructDef *r);
+PLUG_API int EquivStruct(StructDef *fil, StructDef *mem);
 
 /* Adjust the address to a multiple of the alignment (no-op if <2).  */
-extern long AlignAdjust(long address, int alignment);
+PLUG_API long AlignAdjust(long address, int alignment);
 
 /* StructName works for disk or memory based StructDefs,
    CopyStruct returns the StructDef in file->structTable equivalent
                to the given base, creating it if necessary */
-extern char *StructName(StructDef *base);
-extern StructDef *CopyStruct(IOStream *file, StructDef *base);
+PLUG_API char *StructName(StructDef *base);
+PLUG_API StructDef *CopyStruct(IOStream *file, StructDef *base);
 
 /*--------------------------------------------------------------------------*/
 
@@ -534,7 +555,7 @@ extern StructDef *CopyStruct(IOStream *file, StructDef *base);
    if n is 0, name is presumed 0-terminated.
    If file->history!=0, the struct is added to the history child.
    This is illegal after the second history record has begun.  */
-extern StructDef *AddStruct(IOStream *file, const char *name, long n);
+PLUG_API StructDef *AddStruct(IOStream *file, const char *name, long n);
 
 /* AddMember increments references to memType, dims -- offset
    is computed if input offset<0.  Returns 0 on success, 1 if name is a
@@ -547,8 +568,8 @@ extern StructDef *AddStruct(IOStream *file, const char *name, long n);
    *******
    WARNING-- if offset is specified, base->alignment cannot be updated,
              and the proper alignment must be set by hand  */
-extern int AddMember(StructDef *base, long offset, const char *name,
-		     StructDef *memType, Dimension *dims);
+PLUG_API int AddMember(StructDef *base, long offset, const char *name,
+                     StructDef *memType, Dimension *dims);
 
 /* DefinePrimitive is called in lieu of AddMember to define a primitive
    data type after AddStruct has been called.  If model==0, then must
@@ -561,15 +582,15 @@ extern int AddMember(StructDef *base, long offset, const char *name,
    If non-zero, fpLayout is copied.
    Returns 0 on success,
            1 on attempt to redefine char, short, int, long, float, or double
-	     to a model other than the default one
+             to a model other than the default one
            2 on attempt to alias another type to string or pointer without
-	     providing an explicit Converter
-	   3 if an opaque type model size differs and no Converter supplied,
-	   4 if Converter supplied without a model
-	   5 if no model supplied for opaque primitive */
-extern int DefinePrimitive(StructDef *base, long size, int alignment,
-			   int addressType, int order, FPLayout *fpLayout,
-			   StructDef *model, Converter *Convert);
+             providing an explicit Converter
+           3 if an opaque type model size differs and no Converter supplied,
+           4 if Converter supplied without a model
+           5 if no model supplied for opaque primitive */
+PLUG_API int DefinePrimitive(StructDef *base, long size, int alignment,
+                           int addressType, int order, FPLayout *fpLayout,
+                           StructDef *model, Converter *Convert);
 
 /* InstallStruct finishes a structure defintion begun with AddStruct
    and AddMember calls.  If base->file==0, base is added to yStructTable
@@ -583,11 +604,11 @@ extern int DefinePrimitive(StructDef *base, long size, int alignment,
    is constructed and installed on the spot.
    In all cases, base->dataOps is set to "validate" the StructDef.
    If model is non-zero, it is assumed to be the correct model for base.  */
-extern void InstallStruct(StructDef *base, StructDef *model);
+PLUG_API void InstallStruct(StructDef *base, StructDef *model);
 
 /* If DefInstallHook is non-0, InstallStruct calls it after
    installing base in yStructTable if base->file!=0 */
-extern void (*DefInstallHook)(const char *name, StructDef *base);
+PLUG_API void (*DefInstallHook)(const char *name, StructDef *base);
 
 /* AddVariable increments references to varType, dims -- address
    is computed if input address<0.  Returns 0 on success, 1 if name is
@@ -601,26 +622,26 @@ extern void (*DefInstallHook)(const char *name, StructDef *base);
    If file->history!=0, the variable is added to the history child
    (i.e.- it becomes a record variable).  This is illegal after the
    second history record has begun.  */
-extern int AddVariable(IOStream *file, long address, const char *name,
-		       StructDef *varType, Dimension *dims);
+PLUG_API int AddVariable(IOStream *file, long address, const char *name,
+                       StructDef *varType, Dimension *dims);
 
 /* AddHistory adds a history child to an existing IOStream, returning the
    file->history if successful.  Size is the approximate maximum size of
    one file in the history family, or 0 for the default (4 Megabytes).  */
-extern HistoryInfo *AddHistory(IOStream *file, long size);
+PLUG_API HistoryInfo *AddHistory(IOStream *file, long size);
 
 /* AddRecord adds a record to the HistoryInfo, returning 0 on success.
    flags is +1 if time exists, +2 if ncyc exists
    The record is assumed to be in the current history child at the
    specified address, or at the next available address if address<0.  */
-extern int AddRecord(HistoryInfo *history,
-		     int flags, double time, long ncyc, long address);
+PLUG_API int AddRecord(HistoryInfo *history,
+                     int flags, double time, long ncyc, long address);
 
 /* AddNextFile adds a new file to the HistoryInfo, returning 0 on
    success.  If the fullname is 0, the NextFileName function is used.
    If the create flag is non-0, the file is created (possibly copying
    the parent data), otherwise the file is expected to exist.  */
-extern int AddNextFile(HistoryInfo *history, char *filename, int create);
+PLUG_API int AddNextFile(HistoryInfo *history, char *filename, int create);
 
 /* Return value must be freed with p_free.
    Algorithm:
@@ -633,40 +654,40 @@ extern int AddNextFile(HistoryInfo *history, char *filename, int create);
    (4) Loop:
          If the increment character is before the beginning of name,
             or if it is not in the range A-Z, a-z, or 0-9, insert the
-	    character "A" and break out of this loop.
+            character "A" and break out of this loop.
          If it is 9 or z or Z, it becomes 0,
-	 else increment it and break out of this loop.
-	 Back up so the increment character becomes the previous character.
+         else increment it and break out of this loop.
+         Back up so the increment character becomes the previous character.
  */
-extern char *NextFileName(const char *name);
+PLUG_API char *NextFileName(const char *name);
 
 /* JumpToTime (JumpToCycle) jumps to the nearest time (ncyc) in the
    current record sequence to the specified time (ncyc).  Returns 0
    on success, non-0 on failure.  */
-extern int JumpToTime(HistoryInfo *history, double time);
-extern int JumpToCycle(HistoryInfo *history, long ncyc);
+PLUG_API int JumpToTime(HistoryInfo *history, double time);
+PLUG_API int JumpToCycle(HistoryInfo *history, long ncyc);
 /* JumpRecord returns 0 on success, 1 on recNumber out of range.
    If recNumber is out of range, there is no current record.  */
-extern int JumpRecord(HistoryInfo *history, long recNumber);
+PLUG_API int JumpRecord(HistoryInfo *history, long recNumber);
 
 /*--------------------------------------------------------------------------*/
 
 /* Parameters for adjusting caching system -- see cache.c */
-extern long yMaxBlockSize, yCacheSize, yCacheTotal;
-extern int yCacheNumber, yCacheN;
+PLUG_API long yMaxBlockSize, yCacheSize, yCacheTotal;
+PLUG_API int yCacheNumber, yCacheN;
 
 /* copy src from 0 to src->nextAddress to dst, return 0 on success */
-extern int YCopyFile(IOStream *dst, IOStream *src);
+PLUG_API int YCopyFile(IOStream *dst, IOStream *src);
 
 /* FlushFile flushes all cache blocks associated with file, optionally
    discarding them altogether.  */
-extern void FlushFile(IOStream *file, int discardCache);
+PLUG_API void FlushFile(IOStream *file, int discardCache);
 
 /* YcRead and YcWrite are the lowest level read and write primitives for
    moving data to or from an IOStream.  The addr and len are in bytes.
    All data goes through the cache buffer system defined in cache.c.  */
-extern long YcRead(IOStream *file, void *buf, long addr, long len);
-extern void YcWrite(IOStream *file, const void *buf, long addr, long len);
+PLUG_API long YcRead(IOStream *file, void *buf, long addr, long len);
+PLUG_API void YcWrite(IOStream *file, const void *buf, long addr, long len);
 
 /* YRead and YWrite are the high level interface to an IOStream.
    The base->file MUST be non-zero.  In both cases, the void* is an
@@ -677,55 +698,55 @@ extern void YcWrite(IOStream *file, const void *buf, long addr, long len);
    in memory.)
    These routines perform any data type conversions necessary to move
    foreign formats, strings, or pointers to and from disk.  */
-extern void YWrite(void *src, long dst, StructDef *base, long number,
-		   const Strider *strider);
-extern void YRead(void *dst, long src, StructDef *base, long number,
-		  const Strider *strider);
+PLUG_API void YWrite(void *src, long dst, StructDef *base, long number,
+                   const Strider *strider);
+PLUG_API void YRead(void *dst, long src, StructDef *base, long number,
+                  const Strider *strider);
 
 /*--------------------------------------------------------------------------*/
 
 /* CLupdate ensures that file->contentsLog exists and is up to date.  */
-extern void CLupdate(IOStream *file);
+PLUG_API void CLupdate(IOStream *file);
 
 /* CLopen checks to see if file has a Clog description appended to it,
    returning 0 if it does, in which case the file is updated.  */
-extern int CLopen(IOStream *file, int familyOK);
+PLUG_API int CLopen(IOStream *file, int familyOK);
 
 /* CLclose is suitable for use as a CloseHook -- appends Clog description
    to end of file and zaps file->contentsLog.  */
-extern void CLclose(IOStream *file);
+PLUG_API void CLclose(IOStream *file);
 
 /* ZapClogFile frees file->contentsLog, and destroys the associated file.  */
-extern void ZapClogFile(IOStream *file);
+PLUG_API void ZapClogFile(IOStream *file);
 
 /* FreeClogFile frees file->contentsLog.  */
-extern void FreeClogFile(IOStream *file);
+PLUG_API void FreeClogFile(IOStream *file);
 
 /* DumpClogFile creates a Contents Log named clogName, describing file.
    This is completely independent of the file->contentsLog, if any.  */
-extern int DumpClogFile(IOStream *file, const char *clogName);
+PLUG_API int DumpClogFile(IOStream *file, const char *clogName);
 
 /* ReadClogFile reads a Contents Log named clogName and makes file
    look like the description therein.  (CLopen with a separate file.)
    This is completely independent of the file->contentsLog, if any.  */
-extern int ReadClogFile(IOStream *file, const char *clogName);
+PLUG_API int ReadClogFile(IOStream *file, const char *clogName);
 
 /* functions to manage file->pointeeList */
-extern void ReadPointees(IOStream *file);
-extern void WritePointees(IOStream *file);
-extern void ClearPointees(IOStream *file, int writing);
+PLUG_API void ReadPointees(IOStream *file);
+PLUG_API void WritePointees(IOStream *file);
+PLUG_API void ClearPointees(IOStream *file, int writing);
 
 /* functions to manage FPLayouts */
-extern FPLayout *MakeFPLayout(FPLayout *model, long size);
-extern void FreeFPLayout(FPLayout *layout);
-extern int SameFPLayout(FPLayout *l, FPLayout *m);
+PLUG_API FPLayout *MakeFPLayout(FPLayout *model, long size);
+PLUG_API void FreeFPLayout(FPLayout *layout);
+PLUG_API int SameFPLayout(FPLayout *l, FPLayout *m);
 
 /* ------------------------------------------------------------------------ */
 
 /* YError and YWarning must be supplied by user of binary file package.
    YError should longjump or exit, YWarning may return.  */
-extern void YWarning(const char *msg);
-extern void YError(const char *msg);
+PLUG_API void YWarning(const char *msg);
+PLUG_API void YError(const char *msg);
 
 /* ------------------------------------------------------------------------ */
 
