@@ -1,15 +1,24 @@
 /*
-    YDATA.H
-    Declare structures and functions for Yorick's "private" data.
+ * $Id: ydata.h,v 1.8 2010/04/16 05:23:43 dhmunro Exp $
+ * Declare structures and functions for Yorick's "private" data.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
-    $Id: ydata.h,v 1.1 1993/08/27 18:32:09 munro Exp $
+/* use the functions declared in yapi.h to retrieve information
+ * for use by new built-in functions - the functions and data
+ * structures here are for internal use by the interpreter, and
+ * are subject to change as the interpreter evolves
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #ifndef YDATA_H
 #define YDATA_H
 
+#include "yapi.h"
+
 #include "hash.h"
 #include "binio.h"
 
@@ -38,7 +47,7 @@
    table for an int and the Add member of the Operations table for a
    double will point to different functions.
 
-   A Symbol represents an variable or an intermediate result in an
+   A Symbol represents a variable or an intermediate result in an
    expression evaluation; Yorick's global symtab and program stack
    consist of Symbols.  Symbols are distinct from DataBlocks in an
    attempt to speed up simple operations on common data types (that is,
@@ -138,17 +147,17 @@ struct Symbol {
     LValues are used for 3 purposes:
     (1) As a temporary describing the result of an indexing operation.
         Further indexing, or other operations not immediately
-	requiring data, may then procede without actually fetching
-	the data.  This is crucial for extracting parts of complicated
-	data structures (especially data in disk files), which
-	often requires several member extractions and/or subarray
-	specifications to get to the interesting data.
+        requiring data, may then procede without actually fetching
+        the data.  This is crucial for extracting parts of complicated
+        data structures (especially data in disk files), which
+        often requires several member extractions and/or subarray
+        specifications to get to the interesting data.
     (2) As an Array with "remote" data -- the result of a reshape
         operation.  This is Yorick's version of FORTRAN equivalence
-	statements.
+        statements.
     (3) To point to data not owned by the Yorick interpreter --
         for example, a global data structure from a compiled routine,
-	like a FORTRAN common block.
+        like a FORTRAN common block.
  */
 typedef struct LValue LValue;
 struct LValue {
@@ -173,12 +182,13 @@ struct Function {
   int references;     /* reference counter */
   Operations *ops;    /* pointer to virtual functions */
   Symbol *constantTable;  /* constants used by this function */
-  long nConstants;        /* length of coonstantTable */
+  long nConstants;        /* length of constantTable */
   int nReq;               /* worst case number of stack elements required */
   int nPos, nKey, nLocal; /* number of positionals, keywords, and locals */
   long hasPosList;        /* bit 0- 0 unless declared with .. parameter
-			     bits 1-30 set if that positional parameter
-			     marked as an output */
+                             bits 1-30 set if that positional parameter
+                             marked as an output */
+  int errup;              /* to mark func to enter caller for dbug */
   Instruction code[1];    /* virtual machine instructions begin here */
   /* First 1+nPos+hasPosList+nKey+nLocal instructions are code[i].index
      for function name (in definition), positional parameters,
@@ -205,6 +215,17 @@ struct BIFunction {
   long index;         /* to globTab -- shorthand for function name */
 };
 
+/* Auto-loaded functions are special interpreted functions which
+   trigger parsing of their source file when first used.  */
+typedef struct autoload_t autoload_t;
+struct autoload_t {
+  int references;      /* reference counter */
+  Operations *ops;     /* virtual function table */
+  long ifile;          /* index into table of autoload files */
+  long isymbol;        /* global symtab index */
+  autoload_t *next;    /* linked list for each ifile */
+};
+
 /* TextStream is a "foreign" data block defined in ascio.c */
 typedef struct TextStream TextStream;
 
@@ -222,12 +243,12 @@ struct Range {
   Operations *ops;    /* virtual function table */
   long min, max, inc; /* min:max:inc, inc guaranteed non-zero */
   int nilFlags;       /* + 1 if min is nil (:N)
-			 + 2 if max is nil (N:)
-			     Note: if inc is nil, inc==1
-			 + 4 if marked index (+:min:max:inc)
-			 + 8 if pseudo index (-:min:max:inc)
-			 +16 if rubber index (..)
-			 +32 if nullifying index, e.g.-  where(0)  */
+                         + 2 if max is nil (N:)
+                             Note: if inc is nil, inc==1
+                         + 4 if marked index (+:min:max:inc)
+                         + 8 if pseudo index (-:min:max:inc)
+                         +16 if rubber index (..)
+                         +32 if nullifying index, e.g.-  where(0)  */
 #define R_MINNIL 1
 #define R_MAXNIL 2
 #define R_MARKED 4
@@ -270,22 +291,22 @@ struct OpTable {  /* virtual function table for Symbol */
 };
 
 /* Virtual function tables for the 5 Symbol types: */
-extern OpTable intScalar;
-extern OpTable longScalar;
-extern OpTable doubleScalar;
-extern OpTable dataBlockSym;
-extern OpTable referenceSym;   /* referenceSym is not a "complete" Symbol
-				  in the sense that the parser ensures it
-				  never appears in a binary operation
-			ops          - &referenceSym
-			index        - to globTab entry
-			value.offset - stack offset (for Return only)  */
-extern OpTable returnSym;      /* returnSym is not a "complete" Symbol
-				  in the sense that the parser ensures it
-				  never appears in a binary operation
-			ops      - &returnSym
-			index    - (unused)
-			value.pc - VM program counter (for Return only)  */
+PLUG_API OpTable intScalar;
+PLUG_API OpTable longScalar;
+PLUG_API OpTable doubleScalar;
+PLUG_API OpTable dataBlockSym;
+PLUG_API OpTable referenceSym;   /* referenceSym is not a "complete" Symbol
+                                    in the sense that the parser ensures it
+                                    never appears in a binary operation
+                        ops          - &referenceSym
+                        index        - to globTab entry
+                        value.offset - stack offset (for Return only)  */
+PLUG_API OpTable returnSym;      /* returnSym is not a "complete" Symbol
+                                    in the sense that the parser ensures it
+                                    never appears in a binary operation
+                        ops      - &returnSym
+                        index    - (unused)
+                        value.pc - VM program counter (for Return only)  */
 /* Keywords may also appear on the program stack--
    these are marked by ops==0.  */
 
@@ -315,7 +336,7 @@ typedef void MemberOp(Operand *op, char *name);
 
 struct Operations {                 /* virtual function table for DataBlock */
   void (*Free)(void *);  /* crucial member for Unref -- first in struct
-			    to allow alternate Operations to be used */
+                            to allow alternate Operations to be used */
   int typeID;     /* unique type ID number */
   int isArray;    /* 1 if this is one of the Array DataBlocks */
   int promoteID;  /* index into Promote array (0-7, 7 means illegal) */
@@ -335,35 +356,50 @@ struct Operations {                 /* virtual function table for DataBlock */
 };
 
 /* Virtual function tables for the DataBlock types: */
-extern Operations charOps;
-extern Operations shortOps;
-extern Operations intOps;
-extern Operations longOps;
-extern Operations floatOps;
-extern Operations doubleOps;
-extern Operations complexOps;
-extern Operations stringOps;
-extern Operations pointerOps;
-extern Operations structOps;
+PLUG_API Operations charOps;
+PLUG_API Operations shortOps;
+PLUG_API Operations intOps;
+PLUG_API Operations longOps;
+PLUG_API Operations floatOps;
+PLUG_API Operations doubleOps;
+PLUG_API Operations complexOps;
+PLUG_API Operations stringOps;
+PLUG_API Operations pointerOps;
+PLUG_API Operations structOps;
 
-extern Operations rangeOps;
-extern Operations lvalueOps;
-extern Operations voidOps;
-extern Operations functionOps;
-extern Operations builtinOps;
-extern Operations structDefOps;
-extern Operations streamOps;
+PLUG_API Operations rangeOps;
+PLUG_API Operations lvalueOps;
+PLUG_API Operations voidOps;
+PLUG_API Operations functionOps;
+PLUG_API Operations builtinOps;
+PLUG_API Operations structDefOps;
+PLUG_API Operations streamOps;
+PLUG_API Operations textOps;
+PLUG_API Operations listOps;
+PLUG_API Operations auto_ops;
+
+/* generic operators are needed to implement foreign objects */
+PLUG_API UnaryOp ComplementX, NegateX, NotX, TrueX, ToAnyX, EvalX, SetupX;
+PLUG_API BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
+PLUG_API BinaryOp GreaterX, GreaterEQX, EqualX, NotEqualX, AssignX;
+PLUG_API BinaryOp OrX, AndX, XorX, ShiftLX, ShiftRX, MatMultX;
+PLUG_API MemberOp GetMemberX;
+PLUG_API PromoteOp PromXX;
+PLUG_API UnaryOp PrintX;
+PLUG_API UnaryOp y_setup_func_hack; /* do not use y_setup_func_hack */
 
 /*--------------------------------------------------------------------------*/
 
-extern DataBlock nilDB;       /* Nil, or [], the one instance of a void.  */
+PLUG_API DataBlock nilDB;       /* Nil, or [], the one instance of a void.  */
 
-extern Instruction *pc;       /* virtual machine program counter */
-extern Symbol *sp;            /* virtual machine stack pointer */
-extern Symbol *spBottom;      /* current bottom of stack */
-extern HashTable globalTable; /* hash table for globTab symbols */
-extern Symbol *globTab;       /* global symbol table, contains any
-			         variable referenced by a Function */
+PLUG_API Instruction *pc;       /* virtual machine program counter */
+PLUG_API Symbol *sp;            /* virtual machine stack pointer */
+PLUG_API Symbol *spBottom;      /* current bottom of stack */
+PLUG_API HashTable globalTable; /* hash table for globTab symbols */
+PLUG_API Symbol *globTab;       /* global symbol table, contains any
+                                   variable referenced by a Function */
+
+PLUG_API Function *y_idler_function;  /* used by yorick-gl */
 
 /*--------------------------------------------------------------------------*/
 
@@ -393,104 +429,98 @@ extern Symbol *globTab;       /* global symbol table, contains any
 
 /*--------------------------------------------------------------------------*/
 
-extern LValue *NewLValueD(long address, StructDef *base, Dimension *dims);
-extern LValue *NewLValueM(Array *owner, void *address,
-			  StructDef *base, Dimension *dims);
-extern void FreeLValue(void *lvalue);     /* *** Use Unref(lvalue) *** */
+PLUG_API LValue *NewLValueD(long address, StructDef *base, Dimension *dims);
+PLUG_API LValue *NewLValueM(Array *owner, void *address,
+                            StructDef *base, Dimension *dims);
+PLUG_API void FreeLValue(void *lvalue);     /* *** Use Unref(lvalue) *** */
 
-extern Function *NewFunction(Symbol *consts, long nConsts, int nPos, int nKey,
-			     int nLocal, long hasPL, int maxStackDepth,
-			     Instruction *code, long codeSize);
-extern void FreeFunction(void *func);       /* *** Use Unref(func) *** */
+PLUG_API Function *NewFunction(Symbol *consts, long nConsts,int nPos,int nKey,
+                               int nLocal, long hasPL, int maxStackDepth,
+                               Instruction *code, long codeSize);
+PLUG_API void FreeFunction(void *func);       /* *** Use Unref(func) *** */
 
-extern Range *NewRange(long min, long max, long inc, int nilFlags);
-extern void FreeRange(void *range);        /* *** Use Unref(range) *** */
+PLUG_API Range *NewRange(long min, long max, long inc, int nilFlags);
+PLUG_API void FreeRange(void *range);        /* *** Use Unref(range) *** */
 
-extern BIFunction *NewBIFunction(BuiltIn *bi, long index);
-extern void FreeBIFunction(void *bif);    /* *** Use Unref(bif) *** */
+PLUG_API BIFunction *NewBIFunction(BuiltIn *bi, long index);
+PLUG_API void FreeBIFunction(void *bif);    /* *** Use Unref(bif) *** */
+
+PLUG_API int yDebugLevel;
 
 /*--------------------------------------------------------------------------*/
+/* following are deprecated -- use alternatives in yapi.h */
 
-extern int yDebugLevel;
-
-extern long Globalize(const char *name, long n);
-extern long GlobalizeDB(const char *name, long n, void *db);
+PLUG_API long Globalize(const char *name, long n);
+PLUG_API long GlobalizeDB(const char *name, long n, void *db);
 
 /* CheckStack ensures that at least n more elements are available at
    the top of the virtual machine stack.  It returns 1 if the stack had
    to be copied to get more space (so that sp changed), else 0.  */
-extern int CheckStack(int n);
-extern void PushIntValue(int i);
-extern void PushLongValue(long l);
-extern void PushDoubleValue(double d);
-extern int PushCopy(Symbol *s);  /* returns 1 if s is DataBlock, else 0 */
-extern void *PushDataBlock(void *db);  /* returns db */
-extern void Drop(int n);
-extern void PopTo(Symbol *s);
+PLUG_API int CheckStack(int n);
+PLUG_API void PushIntValue(int i);
+PLUG_API void PushLongValue(long l);
+PLUG_API void PushDoubleValue(double d);
+PLUG_API int PushCopy(Symbol *s);  /* returns 1 if s is DataBlock, else 0 */
+PLUG_API void *PushDataBlock(void *db);  /* returns db */
+PLUG_API void Drop(int n);
+PLUG_API void PopTo(Symbol *s);
 
-void ReplaceRef(Symbol *stack);
+PLUG_API void ReplaceRef(Symbol *stack);
+PLUG_API DataBlock *ForceToDB(Symbol *s);
 
 /* Conform sets 4 bit if not conformable, sets 1 bit if ldims==1 where
    rdims>1, sets 2 bit if rdims==1 where ldims>1.  The result dimension
    list is returned in tmpDims.  */
-extern int Conform(Dimension *ldims, Dimension *rdims);
+PLUG_API int Conform(Dimension *ldims, Dimension *rdims);
 /* BinaryConform assures that the operands are conformable; either or
    both may be broadcast.  Result dimensions are left in tmpDims.
    RightConform is the same except an error is signaled if ldims would
    need to be broadcast.
    Both functions return 0 on success, 1 on failure.   */
-extern int BinaryConform(Operand *l, Operand *r);
-extern int RightConform(Dimension *ldims, Operand *r);
+PLUG_API int BinaryConform(Operand *l, Operand *r);
+PLUG_API int RightConform(Dimension *ldims, Operand *r);
 
-extern Array *FetchLValue(void *db, Symbol *dsts);
-extern void StoreLValue(void *db, void *data);
+PLUG_API Array *FetchLValue(void *db, Symbol *dsts);
+PLUG_API void StoreLValue(void *db, void *data);
 
 /*--------------------------------------------------------------------------*/
 
-extern BuiltIn *yBuiltIns[];
-extern void *yGlobals[];
-extern char *yLinkNames[];
-extern char *initialIncludes[];
-extern void YorickInit(BuiltIn **, void **, char **, char **);
-extern int ym_argc;
-extern char **ym_argv;
+PLUG_API void PushTask(Function *task);
+PLUG_API void RunTaskNow(Function *task);
 
-extern void ym_queue(Function *task);
-extern void RunTaskNow(Function *task);
-
-extern int CalledAsSubroutine(void);
+PLUG_API int CalledAsSubroutine(void);
 
 /* Extract scalar integers, reals, and strings from the stack */
-extern long YGetInteger(Symbol *s);
-extern double YGetReal(Symbol *s);
-extern char *YGetString(Symbol *s);
+PLUG_API long YGetInteger(Symbol *s);
+PLUG_API double YGetReal(Symbol *s);
+PLUG_API char *YGetString(Symbol *s);
 
-extern int YNotNil(Symbol *s);
+PLUG_API int YNotNil(Symbol *s);
 
 /* Scan stack[0],...,stack[nArgs-1] for keywords with names in the
    0-terminated list keyNames, putting the corresponding Symbol*s in
    the array symbols, or 0 if the keyName was not found.  Returns the
    Symbol* of the first non-keyword in the input stack list, or 0 if
    none.  */
-extern Symbol *YGetKeywords(Symbol *stack, int nArgs, char **keyNames,
-			    Symbol **symbols);
+PLUG_API Symbol *YGetKeywords(Symbol *stack, int nArgs, char **keyNames,
+                              Symbol **symbols);
 
-extern IOStream *YGetFile(Symbol *stack);
+PLUG_API IOStream *YGetFile(Symbol *stack);
 
 /* Retrieve array arguments for foreign code wrappers,
    applying type conversion (modifies s) if necessary.
    If dims is non-zero, *dims is set to the Dimension * for the argument.
    -- Just cast YGetInteger, YGetReal for scalar arguments, and
       use YGetString for scalar strings.  */
-extern char *YGet_C(Symbol *s, int nilOK, Dimension **dims);
-extern short *YGet_S(Symbol *s, int nilOK, Dimension **dims);
-extern int *YGet_I(Symbol *s, int nilOK, Dimension **dims);
-extern long *YGet_L(Symbol *s, int nilOK, Dimension **dims);
-extern float *YGet_F(Symbol *s, int nilOK, Dimension **dims);
-extern double *YGet_D(Symbol *s, int nilOK, Dimension **dims);
-extern double *YGet_Z(Symbol *s, int nilOK, Dimension **dims);
-extern char **YGet_Q(Symbol *s, int nilOK, Dimension **dims);
-extern void **YGet_P(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API char *YGet_C(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API short *YGet_S(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API int *YGet_I(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API long *YGet_L(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API float *YGet_F(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API double *YGet_D(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API double *YGet_Z(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API char **YGet_Q(Symbol *s, int nilOK, Dimension **dims);
+PLUG_API void **YGet_P(Symbol *s, int nilOK, Dimension **dims);
 /* Convenience routine YGet_dims converts a Dimension * linked list
    to a dimension list dlist --
    dlist[0] is the first dimension length, dlist[1] the second, and so on.
@@ -499,7 +529,7 @@ extern void **YGet_P(Symbol *s, int nilOK, Dimension **dims);
    at most maxDims elements.
    Use the TotalNumber and CountDims functions to get just the length
    or number of dimensions of an array.  */
-extern int YGet_dims(const Dimension *dims, long *dlist, int maxDims);
+PLUG_API int YGet_dims(const Dimension *dims, long *dlist, int maxDims);
 /* If a compiled function is to treat a parameter as an update or an
    output, call YPut_Result after you call the function.  The required
    index is obtained from YGet_Ref, which must be called BEFORE YGet_...
@@ -508,8 +538,8 @@ extern int YGet_dims(const Dimension *dims, long *dlist, int maxDims);
       double *x= YGet_D(sp, 0, (Dimension **)0);
       your_function(sp);
       YPut_Result(sp, index);                */
-extern long YGet_Ref(Symbol *s);
-extern void YPut_Result(Symbol *s, long index);
+PLUG_API long YGet_Ref(Symbol *s);
+PLUG_API void YPut_Result(Symbol *s, long index);
 
 /* generic temporary object has only a free method (zapper)
  * its data structure must begin with
@@ -521,7 +551,58 @@ extern void YPut_Result(Symbol *s, long index);
  * you can then use PushDataBlock to put it on the stack
  *   where Drop will call your zapper
  */
-extern void *y_new_tmpobj(unsigned long size, void (*zapper)(void *to));
+PLUG_API void *y_new_tmpobj(unsigned long size, void (*zapper)(void *to));
+PLUG_API Operations yo_tmpobj;
+
+/* new improved (1.6) argument retrieval functions (no sp reference)
+ * all return 0 for iarg (=number of spots below top of stack) < 0
+ * need to check yarg_nil separately if required
+ */
+/* note that yarg_c, yarg_s, and yarg_f are nowhere referenced in yorick */
+PLUG_API char *yarg_c(int iarg, Dimension **dims);
+PLUG_API short *yarg_s(int iarg, Dimension **dims);
+PLUG_API int *yarg_i(int iarg, Dimension **dims);
+PLUG_API long *yarg_l(int iarg, Dimension **dims);
+PLUG_API float *yarg_f(int iarg, Dimension **dims);
+PLUG_API double *yarg_d(int iarg, Dimension **dims);
+PLUG_API double *yarg_z(int iarg, Dimension **dims);
+PLUG_API char **yarg_q(int iarg, Dimension **dims);
+PLUG_API void **yarg_p(int iarg, Dimension **dims);
+#define yarg_sc(iarg) ((char)yarg_sl(iarg))
+#define yarg_ss(iarg) ((short)yarg_sl(iarg))
+#define yarg_si(iarg) ((int)yarg_sl(iarg))
+PLUG_API long yarg_sl(int iarg);
+#define yarg_sf(iarg) ((float)yarg_sd(iarg))
+PLUG_API double yarg_sd(int iarg);
+PLUG_API char *yarg_sq(int iarg);
+#define yarg_sp(iarg) (yarg_p(iarg,0)[0])
+PLUG_API IOStream *yarg_file(int iarg);
+PLUG_API Operand *yarg_op(int iarg, Operand *op);
+/* yarg_keys returns next non-keyword iarg, or <0 if none */
+PLUG_API int yarg_keys(int iarg, char **knames, Symbol **ksymbols);
+
+/*--------------------------------------------------------------------------*/
+
+typedef char *y_pkg_t(char ***ifiles, BuiltIn ***code, void ***data,
+                      char ***varname);
+PLUG_API void y_pkg_add(y_pkg_t *init);
+PLUG_API y_pkg_t *y_pkg_lookup(char *name);
+/* use name==0 to link or include all packages (static and dynamic) */
+PLUG_API void y_pkg_link(char *name);
+PLUG_API void y_pkg_include(char *name, int now);
+PLUG_API char *y_pkg_name(int i);
+PLUG_API int y_pkg_count(int i);
+
+/* called by on_launch generated by codger in yinit.c */
+PLUG_API int y_launch(int argc, char *argv[],
+                      char *home, char *site, y_pkg_t **pkgs);
+
+PLUG_API int ym_argc;
+PLUG_API char **ym_argv;
+
+#ifdef Y_FOR_YORAPI
+PLUG_API y_pkg_t yk_yor;
+#endif
 
 /*--------------------------------------------------------------------------*/
 
